'''This file defines a battle environment for IP-Saints'''

import pygame
from spg import gui
from spg import defaultStyle
import gui_extras
import random

from unit              import *
from extended_graphics import *
from pause             import *
from quit              import *
from util              import *

class Grid:
    def __init__(self, tiles):
        self.tiles = tiles

    def isSolid(self, position):
        (x, y) = position
        if x < 0:
            return True
        if x >= 20:
            return True
        if y < 0:
            return True
        if y >= 15:
            return True
        return (self.tiles[y])[x] == "#"

class Battleground:
    def __init__(self, image, grid):
        self.image  = image
        self.grid   = Grid(grid)

    def draw(self, surface):
        surface.blit(self.image, (0,0))

class Battle:
    def __init__(self, battleground, player_units, enemy_units, objectives, script):
        '''A battle environment is instantiated with a battleground, a list of
        player units, a list of enemy units and an objective.'''
        self.battleground = battleground
        self.player_units = player_units
        self.enemy_units  = enemy_units
        self.objectives   = objectives
        self.script       = script

    def drawAll(self, points=[], tile=None, ep = None, extra_tile = None):
        '''Yes, I have desecrated this method over and over.'''
        screen = display.get_surface()
        self.battleground.draw(screen)
        for p in points:
            screen.blit(tile, (p[0]*40, p[1]*40))
        if ep:
            screen.blit(extra_tile, (ep[0]*40, ep[1]*40))
        for u in self.player_units.values() + self.enemy_units.values():
            u.draw(screen)

    def displayObjectives(self, button = None):
        self.drawAll(); gui_extras.alert("Victory conditions", self.objectives)
        unpause() #Keep this here to avoid unwanted consequences from further changes in gui_extras

    def fadeIn(self):
        screen = display.get_surface()
        self.drawAll()
        image = screen.copy()
        fade_in(image)

    def switchTurn(self):
        if self.turn == "player":
            self.turn = "enemy"
        else:
            self.turn = "player"

    def options(self, position):
        clock    = pygame.time.Clock()
        desktop  = gui.Desktop()

        #Close-to-mouse window positioning:
        size     = (200, 200)
        position = (position[0] - size[0]/2, position[1])
        if 800 - position[0] < size[0]:
            position = (position[0] - size[0]/2, position[1])
        if 600 - position[1] < size[1]:
            position = (position[0], position[1] - size[1])
        window  = gui_extras.FixedWindow(position = position, size = size, parent = desktop, text = "Options", closeable = True)
        window.onClose = unpause
        button1 = gui.Button(position = (20, 40), size = (160, 0), parent = window, text = "Objectives")
        button2 = gui.Button(position = (20, 80), size = (160, 0), parent = window, text = "Save game")
        button3 = gui.Button(position = (20, 120), size = (160, 0), parent = window, text = "Leave game")
        button4 = gui.Button(position = (20, 160), size = (160, 0), parent = window, text = "Exit program")
        button1.onClick = self.displayObjectives
        button4.onClick = postQuit

        pause()
        while paused():
            for e in gui.setEvents():
                if e.type == pygame.QUIT:
                    postQuit()

            desktop.update()        
            self.drawAll()
            desktop.draw()
            pygame.display.flip()
            clock.tick(60)

    def clicked_unit(self, position):
        pos = (position[0]//40, position[1]//40)
        for u in self.player_units.values() + self.enemy_units.values():
            if u.position == pos:
                return u

    def kill(self, unit):
        self.script.die(self, unit)
        if unit.player == True:
            self.player_units.pop(unit.name)
        else:
            self.enemy_units.pop(unit.name)
            
        print unit.name, "is dead!"

    def unitAttack(self, unit):
        p = unit.position
        target = None
        screen = pygame.display.get_surface()
        enemy_positions = [u.position for u in self.enemy_units.values()]
        adjacent_points = [(p[0]-1, p[1]), (p[0]+1, p[1]), (p[0], p[1]-1), (p[0], p[1]+1)]
        points = []
        for p in adjacent_points:
            if p in enemy_positions:
                points.append(p)
        tile    = load_image(join("data", "images", "red_tile.png"), alpha = False)
        g_tile  = load_image(join("data", "images", "green_tile.png"), alpha = False)
        clock   = pygame.time.Clock()
        if points:
            pause()
            moved = False
            alpha = 60
            alphaSpeed = 1
            attacked   = False
            while paused():
                for e in gui.setEvents():
                    if e.type == pygame.QUIT:
                        postQuit()
                    if e.type == pygame.MOUSEBUTTONDOWN:
                        position = (e.pos[0]//40, e.pos[1]//40)
                        #I don't care. I'm just writing bad code now. If it runs I'm happy
                        if position == unit.position:
                            unpause()
                        if position in points:
                            for u in self.enemy_units.values():
                                if u.position == position:
                                    target = u
                            unit.attack(target)
                            unpause()

                tile.set_alpha(alpha)
                g_tile.set_alpha(alpha)
                self.drawAll(points, tile, unit.position, g_tile)
                pygame.display.flip()
                clock.tick(60)
                alpha += alphaSpeed
                if alpha >= 120:
                    alphaSpeed = -1
                if alpha <= 60:
                    alphaSpeed = 1

        unit.active = False
        if not unit.isAlive():
            self.kill(unit)
        if target:
            if not target.isAlive():
                self.kill(target)


    def unitMove(self, unit):
        points = flood_fill(unit, unit.position, self, unit.stats["speed"])
        for u in self.player_units.values() + self.enemy_units.values():
            if u != unit and u.position in points:
                points.remove(u.position)
        tile   = load_image(join("data", "images", "green_tile.png"), alpha = False)
        clock   = pygame.time.Clock()

        moved = False
        alpha = 60
        alphaSpeed = 1
        pause()
        while paused():
            for e in gui.setEvents():
                if e.type == pygame.QUIT:
                    postQuit()
                if e.type == pygame.MOUSEBUTTONDOWN:
                    position = (e.pos[0]//40, e.pos[1]//40)
                    if position in points:
                        unit.position = position
                        moved = True
                        unpause()
            tile.set_alpha(alpha)
            self.drawAll(points, tile)
            pygame.display.flip()
            clock.tick(60)
            alpha += alphaSpeed
            if alpha >= 120:
                alphaSpeed = -1
            if alpha <= 60:
                alphaSpeed = 1

        if moved:
            self.unitAttack(unit)

    def playerTurn(self, position, buttons):
        unit = self.clicked_unit(position)
        if unit:
            if unit.player:
                if unit.active:
                    self.unitMove(unit)
                else:
                    unit.statusScreen()
            else:
                unit.statusScreen()
        else:
            self.options(position)

    def setTurn(self, who):
        self.turn = who
        if who == "player":
            for u in self.player_units.values():
                u.activate()
        else:
            for u in self.enemy_units.values():
                u.activate()

    def turnOver(self, who):
        if who == "player":
            for u in self.player_units.values():
                if u.active:
                    return False
        if who == "enemy":
            for u in self.enemy_units.values():
                if u.active:
                    return False
        return True

    def AIControl(self, unit):
        screen = pygame.display.get_surface()
        clock  = pygame.time.Clock()
        r_tile = load_image(join("data", "images", "red_tile.png"), alpha = False)
        g_tile = load_image(join("data", "images", "green_tile.png"), alpha = False)

        #Move
        pos = path_find(self, unit)
        points = flood_fill(unit, unit.position, self, unit.stats["speed"])
        self.drawAll(points, g_tile)
        display.flip()

        alpha = 60
        alphaSpeed = 1

        for t in xrange(0, 75):
            for e in gui.setEvents():
                if e.type == pygame.QUIT:
                    postQuit()
                    return
            g_tile.set_alpha(alpha)
            self.drawAll(points, g_tile)
            pygame.display.flip()
            clock.tick(60)
            alpha += alphaSpeed
            if alpha >= 120:
                alphaSpeed = -1
            if alpha <= 60:
                alphaSpeed = 1

        unit.position = pos

        #Attack
        p = unit.position
        target = None
        enemy_positions = [u.position for u in self.player_units.values()]
        adjacent_points = [(p[0]-1, p[1]), (p[0]+1, p[1]), (p[0], p[1]-1), (p[0], p[1]+1)]
        points = []
        for p in adjacent_points:
            if p in enemy_positions:
                points.append(p)
        if len(points) > 0:
            p = points[random.randint(0, len(points)-1)]
            for u in self.player_units.values():
                if p == u.position:
                    target = u

            for t in xrange(0, 75):
                for e in gui.setEvents():
                    if e.type == pygame.QUIT:
                        postQuit()
                        return
                g_tile.set_alpha(alpha)
                r_tile.set_alpha(alpha)
                self.drawAll(points, r_tile, unit.position, g_tile)
                pygame.display.flip()
                clock.tick(60)
                alpha += alphaSpeed
                if alpha >= 120:
                    alphaSpeed = -1
                if alpha <= 60:
                    alphaSpeed = 1
            unit.attack(target)
            unit.active = False
            if not unit.isAlive():
                self.kill(unit)
            if target:
                if not target.isAlive():
                    self.kill(target)

        #self.drawAll(points, tile, unit.position, g_tile)

    def run(self):
        self.script.setup(self)
        self.fadeIn()
        self.script.begin(self)
        self.displayObjectives()

        clock   = pygame.time.Clock()
        desktop = gui.Desktop()
        desktop.onClick = self.playerTurn
        self.setTurn("player")

        while not self.script.isOver(self) and not hasQuit():
            if self.turn == "player":
                for e in gui.setEvents():
                    if e.type == pygame.QUIT:
                        postQuit()

                desktop.update()
                self.drawAll()
                desktop.draw()
                pygame.display.flip()
                clock.tick(60)
                if not self.script.isOver(self):
                    if self.turnOver("player"):
                        self.setTurn("enemy")

            if self.turn == "enemy":
                for u in self.enemy_units.values():
                    for e in gui.setEvents():
                        if e.type == pygame.QUIT:
                            postQuit()

                    self.AIControl(u)
                self.setTurn("player")
            
        if not hasQuit():
            self.script.end(self)
            fade_out()

