'''This file provides a few extended graphic functions.'''

from pygame import surface, rect, image, display, time, font

SCREEN_W    = 800
SCREEN_H    = 600
SCREEN_SIZE = (SCREEN_W, SCREEN_H)

blit_window_list = {}
image_list = {}
font_list = {}

def fade_in(orig_image, speed = 5):
    '''Starts from a black screen and fades into the given image.'''
    screen = display.get_surface()
    clock  = time.Clock()

    #Needed to make copy of the image, because extended graphics are loading with convert_alpha()
    image  = orig_image.copy().convert()

    for x in xrange(0, 255 + speed, speed):
        image.set_alpha(x)
        screen.fill((0,0,0))
        screen.blit(image, (0,0))
        display.flip()
        clock.tick(60)

def fade_out(speed = 5):
    '''Fades out into a black screen.'''
    screen = display.get_surface()
    clock  = time.Clock()
    image  = screen.copy()

    for x in xrange(255, 0 - speed, -speed):
        image.set_alpha(x)
        screen.fill((0,0,0))
        screen.blit(image, (0,0))
        display.flip()
        clock.tick(60)

def tint_up(image, speed = 5):
    '''Assumes the image is already placed on the screen. Increases its
    brightness.'''
    screen = display.get_surface()
    clock  = time.Clock()

    for x in xrange(180, 255 + speed, speed):
        image.set_alpha(x)
        screen.fill((0,0,0))
        screen.blit(image, (0,0))
        display.flip()
        clock.tick(60)

def tint_down(speed = 5):
    ''''Reduces the brightness of the screen.'''
    screen = display.get_surface()
    clock  = time.Clock()
    image  = screen.copy()

    for x in xrange(255, 140 - speed, -speed):
        image.set_alpha(x)
        screen.fill((0,0,0))
        screen.blit(image, (0,0))
        display.flip()
        clock.tick(60)

    return image

def initialize_graphics():
    display.set_mode(SCREEN_SIZE)

def load_blit_window(file_name):
    '''Loads and returns a blit window from a file. This function keeps track of
    previously loaded blit windows to avoid multiple instances of the exact same
    image.'''
    global blit_window_list
    if not blit_window_list.has_key(file_name):
        blit_window_list[file_name] = image.load(file_name).convert()
    blit_window_list[file_name].set_colorkey((0,0,0))
    return blit_window_list[file_name]

def load_font(file_name, size):
    '''Loads and returns a font from a file. This function keeps track of
    previously loaded fonts to avoid multiple instances of the exact same
    font.'''
    global font_list
    key = file_name+str(size)
    if not font_list.has_key(key):
        font_list[key] = font.Font(file_name, size)
    return font_list[key]
    

def load_image(file_name, alpha = True):
    '''Loads and returns an image from a file. This function keeps track of
    previously loaded images to avoid multiple instances of the exact same
    image.'''
    global image_list
    if not image_list.has_key(file_name):
        if alpha:
            image_list[file_name] = image.load(file_name).convert_alpha()
        else:
            image_list[file_name] = image.load(file_name).convert()
    return image_list[file_name]

