from os.path import *

from extended_graphics import *
from util              import *
from quit              import *

from spg import gui
import gui_extras
import pygame

def card_image(card):
    result = load_image(join("data", "images", "card_button.png")).subsurface((0, 0), (60, 80)).copy()
    cardImages = load_image(join("data", "images", "card_images.png"))
    frame_w = card_index(card) * 40
    result.blit(cardImages, (10, 20), ((frame_w, 0), (40, 40)))
    return result


class Unit:
    def __init__(self, name, portrait, model_small, model_big, player = False, stats = None, position = None):
        self.name        = name
        self.portrait    = portrait
        self.model_small = model_small
        self.model_big   = model_big
        self.position    = position
        self.player      = player
        self.stats       = stats
        #Non-parameter attributes
        self.active = True
        self.frame  = 0
        self.timer  = 0
        self.cards  = []
        self.renew()

    def activate(self):
        self.active = True
        complete_cards(self.cards)
        self.stats["mp"] += 1
        if self.stats["mp"] > self.stats["mp-max"]:
            self.stats["mp"] = self.stats["mp-max"]
        self.stats["hp"] += 3
        if self.stats["hp"] > self.stats["hp-max"]:
            self.stats["hp"] = self.stats["hp-max"]

    def applyDamage(self, quantity):
        self.stats["hp"] -= quantity

    def doAttack(self):
        '''You know all hope is lost when functions receive names like this.'''
        if self.player:
            card = gui_extras.cardSelect(self.cards, None)
        else:
            card = best_attack_card(self.cards)
        if card:
            self.cards.remove(card)
        print "Card is", card
        return card

    def doDefend(self, attack_card):
        '''See function above for more information.'''
        if self.player:
            card = gui_extras.cardSelect(self.cards, attack_card)
        else:
            #TODO: Change to a less stupid card selection method.
            card = best_defense_card(self.cards)
            if attack_wins(attack_card, card) and card != "D":
                card = worst_defense_card(self.cards)
        if card:
            self.cards.remove(card)
        print "Card is", card
        return card

    def defend(self, card):
        defense_card = self.doDefend(card)
        return defense_card

    def animateDodge(self):
        screen = pygame.display.get_surface()
        background = screen.copy()
        pygame.display.flip()
        font = load_font(join("data", "fonts", "PhaistosDisk_-_Layne_Space_Age.ttf"), 15)
        text = self.name + " defends the attack!"
        textImage = font.render(text, True, (255, 255, 255))
        r = textImage.get_rect()
        r.center = (400, 250)
        desktop = gui.Desktop()
        window = gui_extras.FixedWindow(position = (r.left-5, r.top-5), size = (r.width+10, r.height+10), parent = desktop)
        delay(1000)
        desktop.update()
        desktop.draw()
        screen.blit(textImage, r)
        pygame.display.flip()
        delay(2000)

    def animateSpecialAttack(self):
        screen = pygame.display.get_surface()
        background = screen.copy()
        pygame.display.flip()
        font = load_font(join("data", "fonts", "PhaistosDisk_-_Layne_Space_Age.ttf"), 15)
        text = self.name + " uses " + self.stats["special-name"] + "!"
        textImage = font.render(text, True, (255, 255, 255))
        r = textImage.get_rect()
        r.center = (400, 250)
        desktop = gui.Desktop()
        window = gui_extras.FixedWindow(position = (r.left-5, r.top-5), size = (r.width+10, r.height+10), parent = desktop)
        delay(500)
        desktop.update()
        desktop.draw()
        screen.blit(textImage, r)
        pygame.display.flip()
        delay(2000)
        

    def animateAttack(self, card, target, damage):
        '''Love does not exist, but this method does.'''
        screen = pygame.display.get_surface()
        background = screen.copy()
        pygame.display.flip()

        if card == "S":
            self.animateSpecialAttack()
            screen.blit(background, (0,0))

        font = load_font(join("data", "fonts", "PhaistosDisk_-_Layne_Space_Age.ttf"), 15)
        if card == "A":
            text = self.name + " hits " +target.name + " CRITICALLY for " + str(damage) + " damage!"
        else:
            text = self.name + " hits " +target.name + " for " + str(damage) + " damage!"
        textImage = font.render(text, True, (255, 255, 255))
        r = textImage.get_rect()
        r.center = (400, 250)
        desktop = gui.Desktop()
        window = gui_extras.FixedWindow(position = (r.left-5, r.top-5), size = (r.width+10, r.height+10), parent = desktop)
        delay(500)
        desktop.update()
        desktop.draw()
        screen.blit(textImage, r)
        pygame.display.flip()
        delay(2000)

        screen.blit(background, (0, 0))

    def attack(self, target):
        backup_image = tint_down()

        screen = pygame.display.get_surface()
        if self.player == True:
            player_unit = self
            enemy_unit  = target
        else:
            player_unit = target
            enemy_unit  = self

        #Display the combat screen
        tile     = load_image(join("data", "images", "fight_tile.png"))
        r        = tile.get_rect()
        r.center = (400, 300)

        screen.blit(tile, r)
        screen.blit(player_unit.portrait, (175, 300))
        screen.blit(enemy_unit.portrait, (475, 75))

        battle_background = screen.copy()

        if not hasQuit():
            attack_card  = self.doAttack()
            if attack_card: screen.blit(card_image(attack_card), (310, 200))
        if not hasQuit():
            screen.blit(battle_background, (0,0))
            screen.blit(card_image(attack_card), (310, 200))
            defense_card = target.defend(attack_card)
            if defense_card: screen.blit(card_image(defense_card), (400, 200))
        if not hasQuit():
            if attack_wins(attack_card, defense_card) == True:
                damage = damage_formula(self, target, attack_card, defense_card)
                self.animateAttack(attack_card, target, damage)
                target.applyDamage(damage)
            else:
                target.animateDodge()

        if  len(target.cards) > 0 and target.isAlive():
            if not hasQuit():
                screen.blit(battle_background, (0,0))
                attack_card  = target.doAttack()
                screen.blit(card_image(attack_card), (310, 200))
            if not hasQuit():
                screen.blit(battle_background, (0,0))
                screen.blit(card_image(attack_card), (310, 200))
                defense_card = self.defend(attack_card)
                if defense_card:
                    screen.blit(card_image(defense_card), (400, 200))
            if not hasQuit():
                if attack_wins(attack_card, defense_card) == True:
                    damage = damage_formula(target, self, attack_card, defense_card)
                    target.animateAttack(attack_card, self, damage)
                    self.applyDamage(damage)
                else:
                    self.animateDodge()

        tint_up(backup_image)
        

    def draw(self, surface):
        self.timer += 1
        if self.timer >= 20:
            self.timer = 0
            self.frame = (self.frame + 1)%2
        if self.isAlive() and self.position:
            surface.blit(self.model_small, (self.position[0]*40, self.position[1]*40), ((40*self.frame, 0), (40, 40)))

    def isAlive(self):
        return self.stats["hp"] > 0

    def gainXP(self, quantity):
        self.stats["xp"] += quantity
        if self.stats["xp"] > 100:
            self.levelUp()

    def levelUp(self):
        self.stats = self.calcStats(self.stats["level"]+1)

    def renew(self):
        self.stats["hp"] = self.stats["hp-max"]
        self.stats["mp"] = self.stats["mp-max"]
        self.cards = []
        complete_cards(self.cards)

    def calcStats(self):
        pass

    def statusScreen(self):
        gui_extras.showStatus(self)

    def speak(self, text):
        gui_extras.speak(self, text)

########################
# Unit specializations #
########################

class Justice(Unit):
    def __init__(self, level = 1, position = None):
        name        = "Justice"
        portrait    = load_image(join("data", "images", "justice_face.png"))
        model_small = load_image(join("data", "images", "justice_small.png"))
        model_big   = None
        player      = True
        stats       = self.calcStats(level)
        Unit.__init__(self, name, portrait, model_small, model_big, player, stats, position)

    def animateSpecialAttack(self):
        Unit.animateSpecialAttack(self)
        sentences = [
            "Piracy is a crime and you're getting jail time for this!",
            "I declare that circumvention of copy-protection is a crime!",
            "I order you to pay hundreds of thousands of dollars in damages for copyright infingement!",
            "Being dead doesn't protect you from lawsuits!"
        ]
        index = random.randint(0, len(sentences)-1)
        self.speak(sentences[index])

    def calcStats(self, level):
        return {
            "level":   level,
            "attack":  7 + level,
            "defense": 7 + level,
            "speed":   5,
            "law":     5 + level,
            "virtue":  5 + level,
            "wisdom":  5 + level,
            "hp-max":  30+5*level,
            "mp-max":  10+5*level,
            "xp":      0,
            "special": "law",
            "special-name": "Balanced rights"
        }

class Law(Unit):
    def __init__(self, level = 1, position = None):
        name        = "Law"
        portrait    = load_image(join("data", "images", "law_face.png"))
        model_small = load_image(join("data", "images", "law_small.png"))
        model_big   = None
        player      = True
        stats       = self.calcStats(level)
        Unit.__init__(self, name, portrait, model_small, model_big, player, stats, position)

    def animateSpecialAttack(self):
        Unit.animateSpecialAttack(self)
        sentences = [
            "Downloading is the same as stealing. Downloading and owning a gun is akin to armed robbery!",
            "You're not allowed to use our music playing software to build nuclear weaponry!",
            "Your illegal usage of our music to entertain babies disgusts me. Take that off the internet now!"
        ]
        index = random.randint(0, len(sentences)-1)
        self.speak(sentences[index])

    def calcStats(self, level):
        return {
            "level":   level,
            "attack":  7 + level,
            "defense": 5 + level,
            "speed":   5,
            "law":     6 + level,
            "virtue":  4 + level,
            "wisdom":  2 + level,
            "hp-max":  25+5*level,
            "mp-max":  7+4*level,
            "xp"    :  0,
            "special": "law",
            "special-name": "Flawless logic"
        }


class Virtue(Unit):
    def __init__(self, level = 1, position = None):
        name        = "Virtue"
        portrait    = load_image(join("data", "images", "virtue_face.png"))
        model_small = load_image(join("data", "images", "virtue_small.png"))
        model_big   = None
        player      = True
        stats       = self.calcStats(level)
        Unit.__init__(self, name, portrait, model_small, model_big, player, stats, position)

    def animateSpecialAttack(self):
        Unit.animateSpecialAttack(self)
        sentences = [
            "To the untrained eye it may look like we're, like, totally dumb!",
            "Artists need a copyright extension, otherwise their children will starve!",
            "You wouldn't download a car.",
        ]
        index = random.randint(0, len(sentences)-1)
        self.speak(sentences[index])


    def calcStats(self, level):
        return {
            "level":   level,
            "attack":  5 + level,
            "defense": 7 + level,
            "speed":   5,
            "law":     2 + level,
            "virtue":  6 + level,
            "wisdom":  4 + level,
            "hp-max":  25+5*level,
            "mp-max":  7+4*level,
            "xp"    :  0,
            "special": "virtue",
            "special-name": "Inspiring speech"
        }


class Wisdom(Unit):
    def __init__(self, level = 1, position = None):
        name        = "Wisdom"
        portrait    = load_image(join("data", "images", "wisdom_face.png"))
        model_small = load_image(join("data", "images", "wisdom_small.png"))
        model_big   = None
        player      = True
        stats       = self.calcStats(level)
        Unit.__init__(self, name, portrait, model_small, model_big, player, stats, position)

    def animateSpecialAttack(self):
        Unit.animateSpecialAttack(self)
        sentences = [
            "We will give our users exactly what they want: more copy protection!",
            "We have just made your life easier by lowering the prices for quoting from us!",
            "It is a shame that medicine is still unable to cure the Linux(TM) cancer!"
        ]
        index = random.randint(0, len(sentences)-1)
        self.speak(sentences[index])


    def calcStats(self, level):
        return {
            "level":   level,
            "attack":  7 + level,
            "defense": 7 + level,
            "speed":   4,
            "law":     4 + level,
            "virtue":  2 + level,
            "wisdom":  6 + level,
            "hp-max":  25+5*level,
            "mp-max":  7+4*level,
            "xp"    :  0,
            "special": "wisdom",
            "special-name": "Innovative idea"
        }


class Pirate(Unit):
    def __init__(self, name, level, position):
        name        = name
        portrait    = load_image(join("data", "images", "pirate_face.png"))
        model_small = load_image(join("data", "images", "pirate_small.png"))
        model_big   = None
        player      = False
        stats       = self.calcStats(level)
        Unit.__init__(self, name, portrait, model_small, model_big, player, stats, position)

    def calcStats(self, level):
        return {
            "level":   level,
            "attack":  5 + level,
            "defense": 5 + level,
            "speed":   4,
            "law":     3 + level,
            "virtue":  1 + level,
            "wisdom":  1 + level,
            "hp-max":  20+5*level,
            "mp-max":  5*level,
            "xp"    :  0,
            "special": "law",
            "special-name": "Torrent upload"
        }

